function[fdatype]=fda2(x,g,among, maxvar,selected)
%fda2				- stepwise factorial discriminant analysis on PCA scores with verification
%function[res]=fda2(X, group, among, maxscore, selected)
%
%Input arguments:
%---------------
%X: SAISIR matrix of the predictive data set (n x p)
%group : SAISIR vector of group (integer values). Identical numbers mean
%that the observations belong to the same group. 
%among : (integer) Maximal rank (dimension) of PC-score allowed to enter in the model
%maxscore: (integer) Maximum number of scores allowed to enter in the model.
%selected: matlab vector (n x 1) with elements =0 (calibration set), or 1
%(validation set)
%
%Output arguments:
%----------------
%res with fields:
%introduced: rank of the PC scores introduced in the model
%ncorrect: number of correct classifications in the calibration set at each step
%nscorrect: number of correct classifications in the validation (supplementary) set at each step
%beta: projection coefficients such as datafactor=X*beta 
%datafactor: discriminant scores if the calibration set
%centroidfactor: scores of the barycenters (centroids) computed from the
%calibration set
%supscore: discriminant scores if the validation set
%eigenval: eigenvalues of the discriminant analysis
%confusion: confusion matrix (row actual; column predicted) in the
%calibration set
%average: average of the calibration set.  
%sconfusion: confusion matrix (row actual; column predicted) in the
%validation (or "supplementary" set)
%
% Assesses a stepwise factorial discriminant analysis according 
% to Bertrand et al., J of Chemometrics, Vol . 4, 413-427 (1990). 
% the basic idea is to assess a factorial discriminant analysis on the scores of 
% a previous pca. The criterion of score selection is the maximisation of 
% the trace of T-1B.  
% In order to avoid using PC-scores with very small eigenvalues, 
% the input argument "among" gives the maximal dimension to be allowed. 
% "maxscore" indicates the maximal number of scores. 
% "datafactor" corresponds to the final model 
%(with maxscore scores introduced). If one is interested in a more
% economical model, it is easy, looking at the classification, to reduce the
% value in "maxscore" and re-run "fda1". 
%The collection is divided in a calibration and a validation set from the
%elements of the input argument "selected"
%
%%Typical example:
%g=create_group1(data,1,3);%% creation of a grouping from the identifiers
%%names, using characters in position 1 to 3
%%random selection of 1/3 in the validation set
%res=fda2(data,g,10,5,random_select(size(data.d,1),round(size(data.d,1)/3)));
%
%SEE ALSO maha3, maha6, plsda, quaddis, applyfda1, fda1. 




xcal=selectrow(x,selected==0);
group=selectrow(g,selected==0);
xval=selectrow(x,selected==1);
gval=selectrow(g,selected==1);

pcatype=pca(xcal);
sup=applypca(pcatype,xval);
[nsuprow ncol1]=size(sup.d);
[nrow ncol]=size(pcatype.score.d);

C=pcatype.score.d(:,1:among);
Y=normc(C);
K=sup.d(:,1:among);%
YS=normc(K);% normed supplementary obs. 
%bid=Y'*Y
ngroup=max(group.d);
aux=zeros(nrow,ngroup);
for i=1:nrow
   aux(i,group.d(i))=1;
end   
w=sum(aux);
P=diag(1./w)*aux'*Y;% gravity center of the normalised scores
diago=diag(P'*diag(w)*P);
[diago,rank]=sort(-diago);
aux=[];
%size(rank)
step=0;
for i=rank(1:maxvar)'
   aux=[aux i];
   P1=P(:,aux);   
   Y1=Y(:,aux);
   YS1=YS(:,aux);
   xpcatype=xpca(P1);% PCA on the gravity centers   
   S=Y1*xpcatype.eigenvec;% scores of the data 
   J=P1*xpcatype.eigenvec;% scores of the gravity centers
   L=YS1*xpcatype.eigenvec;% score of the sup. data
   %size(J)
   %size(S)
   
   for j=1:nrow
   	delta=J-ones(ngroup,1)*S(j,:);   
      dist=sum((delta.*delta)',1);   
      [aux1 predicted_group]=min(dist);
   	predict(j)=predicted_group;   
   end
   
   for j=1:nsuprow
      delta=J-ones(ngroup,1)*L(j,:);   
      dist=sum((delta.*delta)',1);   
      [aux1 predicted_group]=min(dist);
   	spredict(j)=predicted_group;   
	end      
   
 step=step+1;   
ncorrect.d(step)=sum(predict'==group.d);
nscorrect.d(step)=sum(spredict'==gval.d);
end
ranked=rank(1:maxvar);
V=pcatype.eigenvec.d(:,ranked)*diag(1./sqrt(pcatype.eigenval.d(ranked)))*xpcatype.eigenvec;
fdatype.introduced.d=rank(1:maxvar);
fdatype.introduced.i=pcatype.score.v(rank(1:maxvar),:);
fdatype.introduced.v='introduced component';
fdatype.ncorrect.i=pcatype.score.v(rank(1:maxvar),:);
fdatype.ncorrect.v='number of correct classifications (calibration set)';
fdatype.ncorrect.d=ncorrect.d';

fdatype.nscorrect.i=pcatype.score.v(rank(1:maxvar),:);
fdatype.nscorrect.v='number of correct classifications (validation set)';
fdatype.nscorrect.d=nscorrect.d';

fdatype.beta.d = V/sqrt(nrow);fdatype.beta.i=pcatype.eigenvec.i; 
for i=1:size(V,2)
   chaine=['F' num2str(i) '        '];
   fdatype.beta.v(i,:)=chaine(1:6);
end
fdatype.datafactor.d=S;
fdatype.centroidfactor.d=J;
fdatype.centroidfactor.v=fdatype.beta.v;
fdatype.datafactor.i=pcatype.score.i;
fdatype.datafactor.v=fdatype.beta.v;
fdatype.supscore.d=L;
fdatype.supscore.i=sup.i;
fdatype.supscore.v=fdatype.beta.v;


fdatype.eigenval.d=xpcatype.eigenval;
fdatype.eigenval.i='eigenvalues';
for i=1:ngroup
   chaine=['G' num2str(i) '        '];
   fdatype.centroidfactor.i(i,:)=chaine(1:6);
end

confusion=zeros(ngroup,ngroup);
for i=1:nrow
   confusion(group.d(i),predict(i))=confusion(group.d(i),predict(i))+1;
end
fdatype.confusion=matrix2saisir(confusion);
fdatype.average=pcatype.average;

sconfusion=zeros(ngroup,ngroup);
for i=1:nsuprow
   sconfusion(gval.d(i),spredict(i))=sconfusion(gval.d(i),spredict(i))+1;
end
fdatype.sconfusion=matrix2saisir(sconfusion);

if(isfield(g,'g'));
    fdatype.confusion.i=g.g.i;
    fdatype.confusion.v=g.g.i;
    fdaype.sconfusion.i=g.g.i;
    fdatype.sconfusion.v=g.g.i;
end


